package cli

import "fmt"

// ExpectedParameterError is created when a command line parameter is missing.
type ExpectedParameterError struct {
	Parameter string
}

// UnknownParameterError is created when a command line parameter is not recognized.
type UnknownParameterError struct {
	Parameter string
}

// InvalidValueError is created when a parameter has an invalid value.
type InvalidValueError struct {
	Value     string
	Parameter string
}

// Error generates the string representation.
func (err *ExpectedParameterError) Error() string {
	return fmt.Sprintf("Expected parameter '%s'", err.Parameter)
}

// Error generates the string representation.
func (err *UnknownParameterError) Error() string {
	return fmt.Sprintf("Unknown parameter '%s'", err.Parameter)
}

// Error generates the string representation.
func (err *InvalidValueError) Error() string {
	return fmt.Sprintf("Invalid value '%s' for parameter '%s'", err.Value, err.Parameter)
}
