package core

import (
	"slices"

	"git.urbach.dev/cli/q/src/asm"
	"git.urbach.dev/cli/q/src/cpu"
	"git.urbach.dev/cli/q/src/expression"
	"git.urbach.dev/cli/q/src/x86"
)

// CallExtern calls an external function.
func (f *Function) CallExtern(fn *Function, parameters []*expression.Expression) ([]*Parameter, error) {
	f.DLLs = f.DLLs.Append(fn.Package, fn.Name)

	var pushedRegisters []cpu.Register

	for _, register := range x86.WindowsVolatileRegisters {
		if f.RegisterIsUsed(register) {
			f.Register(asm.PUSH, register)
			f.FreeRegister(register)
			pushedRegisters = append(pushedRegisters, register)
		}
	}

	registers := x86.WindowsInputRegisters[:len(parameters)]
	err := f.ExpressionsToRegisters(parameters, registers, fn.Input, true)

	if err != nil {
		return nil, err
	}

	f.Register(asm.PUSH, x86.R5)
	f.RegisterRegister(asm.MOVE, x86.R5, x86.SP)
	f.RegisterNumber(asm.AND, x86.SP, -16)
	f.Number(asm.PUSH, 0)
	f.Number(asm.PUSH, 0)
	f.RegisterNumber(asm.SUB, x86.SP, 32)
	f.DLLCall(fn.UniqueName)
	f.RegisterRegister(asm.MOVE, x86.SP, x86.R5)
	f.Register(asm.POP, x86.R5)

	for _, register := range registers {
		f.FreeRegister(register)
	}

	for _, register := range slices.Backward(pushedRegisters) {
		f.Register(asm.POP, register)
	}

	return fn.Output, nil
}
